
/**
 * @author Nicolas Barradeau 
 * http://en.nicoptere.net
 * twitter @nicoptere
 */
(function() 
{
	
	var raf = 	window.requestAnimationFrame		||
				window.webkitRequestAnimationFrame 	||
				window.mozRequestAnimationFrame 	||
				window.msRequestAnimationFrame 		||
				window.oRequestAnimationFrame 		||
				function(func) { setTimeout(func, 1000 / 30 ); };
	
	var c;
	var ctx;
	var width;
	var height;
	document.body.onload = function()
	{
		
		c = document.getElementById( "canvas" );
		ctx = c.getContext( "2d" );
		
		window.onresize = onResize;
		window.onmousedown = onMouseDown;
		setTimeout( start, 0 );
  }
  function start()
  {
		onResize();
		reset();
		update();
	}
	
	function reset()
	{
		node.list = [];
		edge.list = [];
		ant.list = [];
		
		var i, j;
		
		for( i = 0; i < 100; i++ )
		{
			
			n = new node( Math.random() * width, Math.random() * height );
			
		}
		
		for( i = 0; i < 3; i++ )
		{
			n = node.list[ Math.floor( Math.random() * node.list.length ) ]
			n.start = true;
		}
    
		for( i = 0; i < 5; i++ )
		{
			n = node.list[ Math.floor( Math.random() * node.list.length ) ]
			n.goal = true;
			n.amount = 10 + Math.random() * 10;
		}
		
		
		var edges = [];
		
		var dist = 0;
		var mindist = 300;
		var indices = new utils.delaunay().compute( node.list );
		
		for( i = 0; i < indices.length; i+=3 )
		{	
			
			var i0 = indices[ i ];
			var i1 = indices[ i+1 ];
			var i2 = indices[ i+2 ];
			var tmp = [i0, i1, i2 ];
			for( j = 0; j < 3; j++ )
			{
				dist = utils.distance( node.list[ tmp[ j ] ], node.list[ tmp[ (j+1)%tmp.length] ] );
				if( dist < mindist ) edges.push( tmp[ j ] ,  tmp[ (j+1)%tmp.length] );
				//edges.push( tmp[ j ] ,  tmp[ (j+1)%tmp.length] );
			}				
		}
		
		for( i = 0; i < edges.length; i+=2 )//build the edges
		{
			new edge( node.list[ edges[ i ] ], node.list[ edges[ i + 1 ] ], .5 );// + Math.random() * 3  );
		}
		
		//creating ants
		for( i = 0; i < 100; i++ )
		{
			
			try{
			var id = Math.floor( Math.random() * node.list.length );
			new ant( node.list[ id ] );
			}catch( err ){}
		}
		
	}
	
	function update()
	{
		ctx.clearRect( 0,0,width,height );
		
		edge.render( ctx );
		
		ctx.fillStyle = '#aaa';
		node.render( ctx );
    
		ctx.fillStyle = '#0f0';
		ant.update( ctx );
		
		raf( update );
	}
	
	function onMouseDown( e )
	{
		reset();
	}
	
	function onResize( e )
	{
		
		width = getWidth( document.body );
		height = getHeight( document.body );
		
		c.width = width;
		c.height = height;
		
	}
	
	function getWidth( element ){return Math.max(element.scrollWidth,element.offsetWidth,element.clientWidth );}
	function getHeight( element ){return Math.max(element.scrollHeight,element.offsetHeight,element.clientHeight );}
	
	
	
	var ant = function( node )
	{
		ant.EXPLORE = "explore";
		ant.RETURN = "return";
		ant.COLLECT = "collect";
		
		
		this.x = 0;
		this.y = 0;
		this.time = 0;
		
		this.node = node;
		
		this.mode = ant.EXPLORE;
		this.edge = this.node.getRandomEdge( this );
		this.next_node = this.edge.getOther( this.node );
		this.loaded = false;
		
		this.reset = function()
		{
			
			this.node = this.next_node;
			this.edge = this.node.getRandomEdge( this );
			this.next_node = this.edge.getOther( this.node );
			this.time = 0;
			
			if( this.next_node.start == true && this.mode == ant.RETURN ) 
			{
				if( this.loaded == true )
				{
					this.edge.thickness += .5;
				}
			}
			else if( this.next_node.start == true && this.mode == ant.EXPLORE ) 
			{
				this.mode = ant.COLLECT;
			}
			//will use the strongest path
			else if( this.node.start == true && this.mode == ant.RETURN ) 
			{
				
				if( this.loaded == true )
				{
					this.node.amount += 1;
				}
				this.loaded = false;
				this.mode = ant.COLLECT;
				
			}
			else if( this.node.goal == false && this.mode == ant.COLLECT )
			{
				if( Math.random() < .05 )
				{
					this.mode = ant.EXPLORE;
				}
				else
				{
					this.mode = ant.COLLECT;
				}
			}
			else if( ( this.node.goal == true && this.mode != ant.RETURN  ) 
			|| this.mode == ant.RETURN )
			{
				
				this.mode = ant.RETURN;
				if( this.loaded == false )
				{
					this.node.amount -= 1;
					if( this.node.amount <= 0 ) 
          {
            this.node.goal = false;
          }
					else this.loaded = true;
				}
			}
			else
			{
				//default : exploration mode
				//randomly picks a path
				this.mode = ant.EXPLORE;
			}
			
			
			
		}
		
		ant.list = ant.list || [];
		ant.list.push( this );
		ant.update = function( ctx )
		{
			ant.list.forEach( function( e,i,a )
			{
				
				ctx.beginPath();
				switch( e.mode )
				{
					case ant.RETURN:
					ctx.fillStyle = "#f00";
          e.edge.thickness += .025;
					break;
					
					case ant.COLLECT:
					ctx.fillStyle = "#FC0";
          e.edge.thickness += .015;
					break;
					
					case ant.EXPLORE:
					ctx.fillStyle = "#00f";
          e.edge.thickness += .01;
					break;
				}
				
				var speed = 2;
				e.time += speed / utils.distance( e.node, e.next_node );
				e.x = utils.lerp( e.time, e.node.x, e.next_node.x );
				e.y = utils.lerp( e.time, e.node.y, e.next_node.y );
				
				
				ctx.arc( e.x, e.y, 2, 0, Math.PI * 2 );
				ctx.fill();
				
				//end node reached ; reset
				if( e.time > 1 )
				{
					e.reset();
				}
				
			} );
			
		}
	}
	
	
	var node = function( x, y )
	{
		this.x = x || 0;
		this.y = y || 0;
		this.start = false;
		this.goal = false;
		this.amount = 10;
		
		node.list = node.list || [];
		node.list.push( this );
		
		this.edges = [];
		
		this.getRandomEdge = function( ant )
		{
			
			var edge;
			
			switch( ant.mode ) 
			{
				//explore
				case ant.EXPLORE:
				
				this.edges.sort( function( a, b ){ return a.thickness>b.thickness?-1:1;} );
				if( Math.random() < .5 )
				{
					edge = this.edges[ 0 ];
				}
				else
				{
					edge = this.edges[ Math.floor( Math.random() * this.edges.length ) ];
				}
				break;
				
				//takes the less marked path
				case ant.RETURN:
				
					edge = this.edges[ Math.floor( Math.random() * this.edges.length ) ];
          
				break;
				
				//or take best candidate
				case ant.COLLECT:
					
					this.edges.sort( function( a, b ){ return a.thickness>b.thickness?1:-1;} );
					edge = this.edges[ 0 ];
					if( edge.thickness < .05 )ant.mode = ant.EXPLORE; 
				break;
				
			}
			
			if( edge==null ) return this.edges[ Math.floor( Math.random() * this.edges.length ) ];
			
			if( this.edges.length > 1 && edge == ant.edge ) return getRandomEdge( ant );
			
			return edge;
				
		}
		
		node.render = function( ctx )
		{
			node.list.forEach(	function(e, i, a ){
				
									ctx.beginPath();
									
									ctx.fillStyle = ( e.goal == true ) ? "rgba(255,0,0,.5)" : ( e.start == true ? "rgba(0,255,0,.5)" : "#888" );
									var radius = ( e.goal == true ) ? e.amount : ( e.start == true ?  e.amount : 3 );
									
									ctx.lineWidth = 1;
									ctx.arc( e.x, e.y, radius, 0, Math.PI * 2 );
									ctx.fill();
									ctx.closePath();
									
								}	);
		}
		
	}
	
	
	var edge = function( node0, node1, thickness )
	{
		this.node0 = node0;
		this.node1 = node1;
		this.thickness = thickness || .1;
		
		if( node0.edges.indexOf( this ) == -1 ) node0.edges.push( this );
		if( node1.edges.indexOf( this ) == -1 ) node1.edges.push( this );
		
		edge.list = edge.list || [];
		edge.list.push( this );
		
		this.contains = function( node )
		{
			return node == this.node0 || node == this.node1;
		}
		
		this.getOther = function( node )
		{
			if( node == this.node0 ) return this.node1;
			if( node == this.node1 ) return this.node0;
			return null;
		}
		
		edge.exist = function( edge )
		{
			return edge.list.indexOf( edge ) != -1;
		}
		
		edge.render = function( ctx )
		{
				
			edge.list.forEach( function (e, i, a )
			{	
				ctx.lineWidth = Math.min( Math.min( 1, Math.max( .01, e.thickness ) )  , 1 );
				ctx.strokeStyle = "rgba( 32,32,32,"+ Math.min( 1, Math.max( .01, e.thickness ) ) * .5 +")";
				
				ctx.beginPath();
				ctx.moveTo( e.node0.x, e.node0.y );
				ctx.lineTo( e.node1.x, e.node1.y );
				ctx.stroke();
				ctx.closePath();
				e.thickness -= 0.015;//evaporation
				e.thickness = Math.min( 10, Math.max( .01, e.thickness ) );
				
			});
			
		}
		
	}
	
	
	var point = function( x,y ){this.x = x||0;this.y = y||0;}
	
	var utils = function(){};
	utils.distance = function( p0, p1 ){return Math.sqrt( ( p0.x - p1.x ) * ( p0.x - p1.x ) + ( p0.y - p1.y ) * ( p0.y - p1.y ) );}
	utils.lerp = function( t, a, b ){ return a + t * ( b - a ); };
	utils.delaunay = function()
	{
		this.EPSILON = 0.0000001;
		this.SUPER_TRIANGLE_RADIUS = 1000000;
		var indices;
		var circles;
		this.compute = function( points )
		{
			points.sort( function( a, b ){ a.x < b.x ? -1 : 1;} );
			with( this )
			{
				indices = [];
				circles = [];
				
				var nv = points.length;
				if (nv < 3) return null;
				
				var d = SUPER_TRIANGLE_RADIUS;
				points.push( 	new point( 0, -d ) 	);
				points.push( 	new point( d, d ) 	);
				points.push( 	new point( -d, d )	);
				
				indices = []; 
				indices.push( points.length - 3 );
				indices.push( points.length - 2 );
				indices.push( points.length - 1 );
				
				this.circles = [];
				this.circles.push( 0 );
				this.circles.push( 0 );
				this.circles.push( d );
				
				var edgeIds = [];
				var i, j, k, id0, id1, id2;
				for ( i = 0; i < nv; i++ )
				{
					
					j = 0;
					while( j < indices.length )
					{
						if ( 	circles[ j + 2 ] > EPSILON 		&& 		circleContains( j, points[ i ] )	)
						{
							id0 = indices[ j ];
							id1 = indices[ j + 1 ];
							id2 = indices[ j + 2 ];
							
							edgeIds.push( id0 );
							edgeIds.push( id1 );
							edgeIds.push( id1 );
							edgeIds.push( id2 );
							edgeIds.push( id2 );
							edgeIds.push( id0 );
							
							indices.splice( j, 3 );
							circles.splice( j, 3 );
							j -= 3;
						}
						j += 3;
					}
					
					j = 0;
					while ( j < edgeIds.length )
					{
						k = ( j + 2 );
						while ( k < edgeIds.length )
						{
							if(	(	edgeIds[ j ] == edgeIds[ k ] && edgeIds[ j + 1 ] == edgeIds[ k + 1 ]	)
							||	(	edgeIds[ j + 1 ] == edgeIds[ k ] && edgeIds[ j ] == edgeIds[ k + 1 ]	)	)
							{
								edgeIds.splice( k, 2 );
								edgeIds.splice( j, 2 );
								j -= 2;
								k -= 2;
								if ( j < 0 || j > edgeIds.length - 1 ) break;
								if ( k < 0 || k > edgeIds.length - 1 ) break;
							}
							k += 2;
						}
						j += 2;
					}
					j = 0;
					while( j < edgeIds.length )
					{
						indices.push( edgeIds[ j ] );
						indices.push( edgeIds[ j + 1 ] );
						indices.push( i );
						computeCircle( points, edgeIds[ j ], edgeIds[ j + 1 ], i );
						j += 2;
					}
					edgeIds = [];
					
				}
				id0 = points.length - 3;
				id1 = points.length - 2;
				id2 = points.length - 1;
				
				i = 0;
				while( i < indices.length )
				{
					if ( indices[ i ] == id0 || indices[ i ] == id1 || indices[ i ] == id2 
					||	 indices[ i + 1 ] == id0 || indices[ i + 1 ] == id1 || indices[ i + 1 ] == id2 
					||	 indices[ i + 2 ] == id0 || indices[ i + 2 ] == id1 || indices[ i + 2 ] == id2 )
					{
						indices.splice( i, 3 );
						circles.splice( i, 3 );
						if( i > 0 ) i-=3;
						continue;
					}
					i += 3;
				}
				
				points.pop();
				points.pop();
				points.pop();
				return indices;
			}
		}
		
		this.circleContains = function( circleId, p )
		{
			var dx = this.circles[ circleId ] - p.x;
			var dy = this.circles[ circleId + 1 ] - p.y;
			return this.circles[ circleId + 2 ] > dx * dx + dy * dy;
		}
		
		this.computeCircle = function( points, id0, id1, id2 )
		{
			var p0 = points[ id0 ];
			var p1 = points[ id1 ];
			var p2 = points[ id2 ];
			var A = p1.x - p0.x;
			var B = p1.y - p0.y;
			var C = p2.x - p0.x;
			var D = p2.y - p0.y;
			var E = A * (p0.x + p1.x) + B * (p0.y + p1.y);
			var F = C * (p0.x + p2.x) + D * (p0.y + p2.y);
			var G = 2.0 * (A * (p2.y - p1.y) - B * (p2.x - p1.x));
			
			var x = (D * E - B * F) / G;
			this.circles.push( x );
			
			var y = (A * F - C * E) / G;
			this.circles.push( y );
			
			x -= p0.x;
			y -= p0.y;
			this.circles.push( x * x + y * y );
		}
		
		this.draw = function( ctx, points, indices, t )
		{
			if( indices == undefined ) return;
			if( t <= 0.01 )return;
			t = t || 1;
			
			var i = 0, id0, id1, id2;
			while ( i < ( indices.length * t )  )
			{
				
				id0 = indices[ i ];
				id1 = indices[ i + 1 ];
				id2 = indices[ i + 2 ];
				
				ctx.beginPath();
				ctx.moveTo( points[ id0 ].x, points[ id0 ].y );
				ctx.lineTo( points[ id1 ].x, points[ id1 ].y );
				ctx.lineTo( points[ id2 ].x, points[ id2 ].y );
				ctx.lineTo( points[ id0 ].x, points[ id0 ].y );
				
				ctx.stroke();
				ctx.closePath();
				i += 3;
				
			}
		}
	}
})();