/**
 * @author Nicolas Barradeau 
 * http://en.nicoptere.net
 * twitter @nicoptere
 */
(function() 
{
	
	/*
	******************************************************
	
	state machine
	
	******************************************************
	*/
	
	//generic State
	var State = function( name ){	this.name 	=	name || "State";}
	State.prototype = {enter : function(){},update : function( time ){},exit : function(){}}
	
	/**
	
	and 4 states of insect locomotion 
	http://www.mindcreators.com/InsectLocomotion.htm
			
	//*/
	var onGround = function(){ this.name = "og"; };
	onGround.prototype = State.prototype;
	
	var onGroundFlexed = function(){ this.name = "ogf"; };
	onGroundFlexed.prototype = State.prototype;
	
	var inAir = function(){ this.name = "ia"; };
	inAir.prototype = State.prototype;
	
	var inAirFlexed = function(){ this.name = "iaf"; };
	inAirFlexed.prototype = State.prototype;
	
	var ON_GROUND = new onGround();
	var ON_GROUND_FLEXED = new onGroundFlexed();
	var IN_AIR = new inAir();
	var IN_AIR_FLEXED = new inAirFlexed();
	
	//the state machine itself
	
	var StateMachine = function( state )
	{
		var previous_state;
		var current_state;
		var next_state;
		
		this.update = function( time ){if( this.current_state != undefined ){	this.current_state.update( time );	}}
		this.setState = function( state )
		{
			if( state == undefined ) return;//nothing to assign
			this.previous_state = this.current_state || state;
			if( this.current_state != undefined )
			{
				if( this.current_state.name === state.name ) return;
				this.current_state.exit();
			}
			this.current_state = state;
			this.current_state.enter();
		}
		
		this.previous = function(){	this.setState( this.previous_state );	}
		this.setNext = function( state ){	this.next_state = state;	}
		this.next = function(){if( this.next_state != undefined ) {this.setState( this.next_state );this.next_state = undefined;}}
		if( state != undefined ) { this.setState( state ); }
		
	}
	
	//generic agent model implementing the FSM
	
	var Agent = function(){}
	Agent.prototype =
	{
		fsm 		: new StateMachine( new State() ),
		update 		: function( time ){	this.fsm.update( time );	},
		setState 	: function( state ){ this.fsm.setState( state ); },
		previous 	: function(){ this.fsm.previous(); },
		setNext 	: function( state ){ this.fsm.setNext( state ); },
		next 		: function(){ this.fsm.next(); },
		getState 	: function(){return this.fsm.current_state;}
	}
	
	/*
	******************************************************
	
	end state machine
	
	******************************************************
	*/
	
	
	
	
	/*
	******************************************************
	
	app
	
	******************************************************
	*/
	
	var raf = 	window.requestAnimationFrame		||
				window.webkitRequestAnimationFrame 	||
				window.mozRequestAnimationFrame 	||
				window.msRequestAnimationFrame 		||
				window.oRequestAnimationFrame 		||
				function(func) { setTimeout(func, 1000 / 30 ); };
	
	var c;
	var ctx;
	var width;
	var height;
	
	var walker;
	
	window.onload = function()
	{
		
		c = document.getElementById( "canvas" );
		ctx = c.getContext( "2d" );
		
		window.onresize = onResize;
		
		time = new Date().getTime() * .005;
		
    
    walker = new Walker( new point( 100,100 ), 6 );
    
		onResize();
		update();
		
	}
	
	
	var Walker = function( position, legCount )
	{
		
		this.position = position;
		this.time = 0;
		this.legs = [];
		
		var i = legCount || 3;
		while( i-- )
		{
			
			var left  = new Leg( this, i % 2 == 0, 	0, Math.PI / 6, 30 );
			var right = new Leg( this, i % 2 != 0,	Math.PI, -Math.PI / 6, 30 );
			
			var p = position.clone();
			
			p.y += 20 + i * 20;
			left.position = p;
			right.position = p.clone();
			this.legs.push( left, right );
			
		}			
		
		this.update = function( time, ctx )
		{
			for( var i = 0; i  < this.legs.length; i++ )
			{
				this.legs[ i ].update( Math.sin( time ), ctx );
			}
		}
		
		this.render = function( time, ctx )
		{
			
			var p;
			
			for( var i = 0; i  < this.legs.length; i+=2 )//reposition legs
			{
				this.legs[ i ].position.x = this.legs[ i+1 ].position.x = this.position.x;
				this.legs[ i ].position.y = this.legs[ i+1 ].position.y = this.position.y + 30 + i * 10;
			}
			
			ctx.fillStyle = "#FFF";
			ctx.strokeStyle = "#333";
			ctx.lineWidth = 3;
			for( var i = 0; i  < this.legs.length; i+=2 )
			{
				p = this.legs[ i ].position;
				ctx.beginPath();
				ctx.arc( p.x, p.y , 5, 0, Math.PI * 2 );
				ctx.stroke();
				ctx.fill();
				ctx.closePath();
			}
			
			
			//big circle
			ctx.lineWidth = 5;
			p = position.clone();
			p.x += Math.random() * Math.min( ( time - this.lastTime ) * 100, 2 );
			p.y += Math.random() * Math.min( ( time - this.lastTime ) * 100, 2 );
			this.lastTime = time;
			
			ctx.beginPath();
			ctx.arc( p.x, p.y , 15, 0, Math.PI * 2 );
			ctx.stroke();
			ctx.fill();
			ctx.closePath();
			
			ctx.lineWidth = 1;
			ctx.beginPath();
			
			var angle = ( time ) * Math.PI;
			ctx.moveTo( p.x + Math.cos( angle ) * 15, p.y + Math.sin( angle ) * 15 );
			ctx.lineTo( p.x + Math.cos( angle + Math.PI ) * 15, p.y + Math.sin( angle + Math.PI ) * 15 );
			ctx.stroke();
			ctx.closePath();
			
			
		}
	}
	
	var Leg = function( body, side, minAngle, maxAngle, length )
	{
		var other;
		var position;
		this.side = side;
		this.body = body;
		this.width = 1;
		this.minAngle = minAngle;
		this.angle = minAngle;
		this.destAngle = minAngle;
		this.maxAngle = maxAngle;
		this.length = length;
		
		this.update = function( time, ctx )
		{
			//symmetry
			if( side == true ) time *= -1;
			
			with( this )
			{
				if( time > 0 )
				{
					setState( ON_GROUND );
					setNext( ON_GROUND_FLEXED );
				}
				else
				{
					setState( IN_AIR_FLEXED );
					setNext( IN_AIR );
				}
				
				angle += ( destAngle - angle ) * .1;
				render( time, ctx );
				
			}
			
		}
		
		this.render = function( time, ctx )
		{
			
			with(this)
			{
				if( getState().name == ON_GROUND.name || getState().name == ON_GROUND_FLEXED.name )
				{
					ctx.strokeStyle = '#00f';
				}
				else
				{
					ctx.strokeStyle = '#f00';
				}
				ctx.lineWidth = 1;
				if( getState().name == IN_AIR_FLEXED.name || getState().name == IN_AIR.name )ctx.lineWidth = 3;
				
				ctx.beginPath();
				ctx.moveTo( position.x, position.y );
				ctx.lineTo( position.x + Math.cos( angle ) * length, position.y + Math.sin( angle ) * length );
				ctx.stroke();
				ctx.closePath();
				
			}
			
		}
		
		this.setState = function( state )
		{
			
			with( this )
			{
				switch( state.name )
				{
					case ON_GROUND.name:
						destAngle = minAngle;
						break;
					
					case ON_GROUND_FLEXED.name:
						destAngle = minAngle + maxAngle;
						break;
					
					case IN_AIR.name:
						destAngle = minAngle;
						break;
					
					case IN_AIR_FLEXED.name:
						destAngle = minAngle + maxAngle;
						break;
				}
				
			}
			Agent.prototype.setState( state );
			
			
		}
		
		this.setOther = function( other ){	this.other = other;	}
		
	}
	Leg.prototype = Agent.prototype;
	
	
	/**********************************************/
	function update()
	{
		ctx.clearRect( 0,0,width,height );
		
		var time = new Date().getTime() * .0025;
		
		walker.update( time, ctx );
		walker.render( time, ctx );
		
		raf( update );
		
	}
	
	function onResize( e )
	{
		
		width = getWidth( document.body );
		height = getHeight( document.body );
		
		c.width = width;
		c.height = height;
		
		walker.position.x = width * .5;
		walker.position.y = height * .5 - 50;
		
	}
	
	function getWidth( element ){return Math.max(element.scrollWidth,element.offsetWidth,element.clientWidth );}
	function getHeight( element ){return Math.max(element.scrollHeight,element.offsetHeight,element.clientHeight );}
	
	
	/*
	******************************************************
	
	geometry utils
	
	******************************************************
	*/
	
	var point = function( x,y ){this.x = x||0;this.y = y||0;}
	point.prototype = { clone : function(){return new point( this.x, this.y );}	}
	
	
})();