/**
 * @author Nicolas Barradeau 
 * http://en.nicoptere.net
 * twitter @nicoptere
 */
(function() 
{
	
	var raf = 	window.requestAnimationFrame		||
				window.webkitRequestAnimationFrame 	||
				window.mozRequestAnimationFrame 	||
				window.msRequestAnimationFrame 		||
				window.oRequestAnimationFrame 		||
				function(func) { setTimeout(func, 1000 / 30 ); };
	
	var c,
		ctx,
		width,
		height,
		mouseX = 0,
		mouseY = 0,
		w2 = 600,
		h2 = 450,
		K = 0.00001,
		radius = 250,
		time = 0.00001,
		tri;
	
	window.onload = function()
	{
		
		c = document.getElementById( "canvas" );
		ctx = c.getContext( "2d" );
		
		
		var p0 = new point(  radius * .75,  radius * .5 );
		var p1 = new point( -radius * .75,  radius * .5 );
		var p2 = new point( 0, -radius * .75 );
		tri = new triangle( p0, p1, p2 );
		
		window.onresize = onResize;
		window.onmousemove = onMouseMove;
		window.onmousedown = reset;
		onResize();
		update();
		
	}
	
	//http://web.engr.oregonstate.edu/~mjb/cs553/Handouts/Hyperbolic/hyperbolic.2pp.pdf
	function reset()
	{
		
		var p0 = new point( radius * ( Math.random() - .5 ) * 2, radius * ( Math.random() - .5 ) * 2 );
		var p1 = new point( radius * ( Math.random() - .5 ) * 2, radius * ( Math.random() - .5 ) * 2 );
		var p2 = new point( radius * ( Math.random() - .5 ) * 2, radius * ( Math.random() - .5 ) * 2 );
		
		tri = new triangle( p0, p1, p2 );
			
	}
	
	function update()
	{
		
		ctx.clearRect( 0,0, width, height );
		
		ctx.strokeStyle = "#AAA";
		ctx.beginPath();
		ctx.arc( w2, h2, radius, 0, Math.PI * 2, true );
		ctx.stroke();
		ctx.closePath();
		
		K = .0001 + 1000 * ( mouseX / width );
		
		if( K > 0 )
		{
			//draws the original triangle
			renderTriangle( tri, "#AAA" );
			
			//draw the hyperbolic triangles using a minimum distance & a recursive function
			var min_dist = .5;
			
			//with a polar method
			hyperTriPolar( 		tri, min_dist, "#F30" );
			
			//with a cartesian method
			hyperTriCartesian( 	tri, min_dist, "#069" );
			
		}
		
		
		ctx.fillStyle = "#999";
		ctx.font = 'normal 12px Verdana';
		ctx.fillText( 'HYPERBOLIC COORDINATES EXAMPLE', 20,20 );
		ctx.fillText( 'click to randomize the triangle', 20,36 );
		ctx.fillText( 'move left & right to change K value (currently: '+K.toFixed( 4 )+' )', 20,52 );
		raf( update );
		
	}
	
	//transforms cartesian coordinates into hyperbolic polar coordinates
	function getHyperbolicPolar( p )
	{
		
		var R = p.length();
		return new point(	( p.x / ( R + K ) ) * radius,
							( p.y / ( R + K ) ) * radius	);
		
	}
	
	//transforms cartesian coordinates into hyperbolic cartesian coordinates
	function getHyperbolicCartesian( p )
	{
		
		var K2 = K * K;
		return new point( 	( p.x / Math.sqrt( p.x * p.x + K2 ) ) * radius,
							( p.y / Math.sqrt( p.y * p.y + K2 ) ) * radius		);
		
	}
	
	function drawLine( p0, p1, minDist, method, strokeStyle )
	{
		
		//default values
		minDist = minDist || .1;
		method = method || getHyperbolicPolar;
		
		//midpoint P0-P1
		var a = new point( p0.x + ( p1.x - p0.x ) / 2 , p0.y + ( p1.y - p0.y ) / 2 );
		
		//projection 
		//origin
		var ha = method( a );
		
		//midpoint
		var hp0 = method( p0 );
		var hp1 = method( p1 );
		var hb = new point( hp0.x + ( hp1.x - hp0.x ) / 2, hp0.y + ( hp1.y - hp0.y ) / 2 );
		
		//distance between the 2 projected points
		var dx = ha.x - hb.x;
		var dy = ha.y - hb.y;
		if( Math.sqrt( dx * dx + dy * dy ) < minDist )
		{
			line( hp0, hp1, strokeStyle );
		}
		else
		{
			drawLine( p0, a, minDist, method, strokeStyle );
			drawLine( a, p1, minDist, method, strokeStyle );
		}
	}
	
	///////////RENDERING
	
	function render( p )
	{
		plot( p, "#F00" );
		plot( getHyperbolicPolar( p ), "#666" );
		plot( getHyperbolicCartesian( p ), "#00f" );
	}
	
	function plot( p, strokeStyle )
	{
		ctx.beginPath();
		ctx.strokeStyle = strokeStyle || "#0F0";
		ctx.arc( w2 + p.x, h2 + p.y, 2, 0, Math.PI * 2, true );
		ctx.stroke();
	}
	
	function line( p0, p1, strokeStyle )
	{
		ctx.beginPath();
		ctx.strokeStyle = strokeStyle || "#00f";
		ctx.moveTo( p0.x + w2, p0.y + h2 );
		ctx.lineTo( p1.x + w2, p1.y + h2 );
		ctx.stroke();
	}
	
	function renderTriangle( tri, strokeStyle )
	{
		ctx.beginPath();
		ctx.strokeStyle = strokeStyle || "#00f";
		ctx.moveTo( tri.p0.x + w2, tri.p0.y + h2 );
		ctx.lineTo( tri.p1.x + w2, tri.p1.y + h2 );
		ctx.lineTo( tri.p2.x + w2, tri.p2.y + h2 );
		ctx.lineTo( tri.p0.x + w2, tri.p0.y + h2 );
		ctx.stroke();
	}
	
	function hyperTriPolar( tri, minDist, strokeStyle )
	{
		drawLine( tri.p0, tri.p1, minDist, getHyperbolicPolar, strokeStyle );
		drawLine( tri.p1, tri.p2, minDist, getHyperbolicPolar, strokeStyle );
		drawLine( tri.p2, tri.p0, minDist, getHyperbolicPolar, strokeStyle );
	}
	
	function hyperTriCartesian( tri, minDist, strokeStyle )
	{
		drawLine( tri.p0, tri.p1, minDist, getHyperbolicCartesian, strokeStyle );
		drawLine( tri.p1, tri.p2, minDist, getHyperbolicCartesian, strokeStyle );
		drawLine( tri.p2, tri.p0, minDist, getHyperbolicCartesian, strokeStyle );
	}
	
	/////// UTILS
	
	function onResize( e )
	{
		c.width = width = getWidth( document.body );
		c.height = height = getHeight( document.body );
		w2 = width / 2;
		h2 = height / 2;
	}
	function getWidth( element ){return Math.max(element.scrollWidth,element.offsetWidth,element.clientWidth );}
	function getHeight( element ){return Math.max(element.scrollHeight,element.offsetHeight,element.clientHeight );}
	
	function onMouseMove(e)
	{
		mouseX = e.clientX;
		mouseY = e.clientY;
	}
	
	var point = function( x,y )
	{
		this.x = x||0;
		this.y = y||0;
		this.clone = function(){return new point( this.x, this.y );}
		this.length = function(){return Math.sqrt( this.x * this.x + this.y * this.y );}
	}
	
	var triangle = function( p0, p1, p2 )
	{ 
		this.p0 = p0||new point();
		this.p1 = p1||new point();
		this.p2 = p2||new point();
	}
	
})();