/**
 * @author Nicolas Barradeau 
 */
	var utils = function(){};
	utils.lerp = function( t, a, b ){ return a + t * ( b - a ); };
	utils.norm = function( t, a, b ){ return ( t - a ) / ( b - a ); };
	utils.map = function( t, a0, b0, a1, b1 ){ return utils.lerp( utils.norm( t, a0, b0 ), a1, b1 ); };
	utils.clamp = function( t, min, max ){ return Math.min( max, Math.max( t, min ) ); };

	var animation = false;
	
	var size = 250;
	var w = 3;
	var h = 3;
	var d = 3;
	
	var gravity = -0.1;
	var friction = 0.8;
	var damping = 0.1;
	
	var bounds = size * .5;
	var bbox = { x:-bounds, y:-bounds, z:-bounds, w:bounds, h:bounds, d:bounds };
	
	var gridGeom,
		gridMat,
		grid;
	
	var sphereGeom = new THREE.IcosahedronGeometry( 1.5,2 );
	var white = new THREE.MeshBasicMaterial({color:0xDDDDDD } );
	var blendings = [ "NoBlending", "NormalBlending", "AdditiveBlending", "SubtractiveBlending", "MultiplyBlending", "AdditiveAlphaBlending" ];
	var blue = new THREE.MeshBasicMaterial({color:0xFF9900});//, transparent: true, opacity: 0.5, blending:THREE[ blendings[ 2 ] ] } );
	
	var force = function( pos, strength )
	{
		this.pos = pos;
		this.acc = new THREE.Vector3(
									( Math.random() - .5 ) * 2,
									( Math.random() - .5 ) * 2,
									( Math.random() - .5 ) * 2
									);
		this.vel = new THREE.Vector3();
		
		this.strength = strength;
		this.time = Math.random();
		
		this.update = function()
		{
			
			this.time += .01;
			
			//this.acc.x += ( ( Math.random() - .5 ) * 2 ) * .01;
			//this.acc.y += ( ( Math.random() - .5 ) * 2 ) * .01;
			//this.acc.z += ( ( Math.random() - .5 ) * 2 ) * .01;
			
			//this.acc.x += ( Math.cos(this.time) ) * .01;
			//this.acc.y += ( Math.sin(this.time) ) * .01;
			//this.acc.z += ( Math.cos(this.time * 2) ) * .01;
			
			this.acc.normalize();
			
			this.vel.x += ( this.acc.x - this.vel.x ) * damping;
			this.vel.y += ( this.acc.y - this.vel.y ) * damping;
			this.vel.z += ( this.acc.z - this.vel.z ) * damping;
			
			//this.vel.add( this.acc );
			
			//this.vel.y = utils.clamp( this.vel.y, -this.strength, this.strength );
			//this.vel.z = utils.clamp( this.vel.z, -this.strength, this.strength );
			//this.vel.x = utils.clamp( this.vel.x, -this.strength, this.strength );
			
			this.vel.multiplyScalar( friction );
			
			this.mesh.scale.copy( this.vel );
			this.mesh.scale.multiplyScalar( this.strength );
			
		}		
		this.mesh = new THREE.Mesh( sphereGeom, blue );
		this.mesh.position.copy( pos );
		this.mesh.scale.multiplyScalar( 20 );
		group.add( this.mesh );
		
		this.dispose = function()
		{
			group.remove( this.mesh );
		}
		this.update();
		return this;
		
	}
	
	////////////////////////////////////////////////
	
	var objects = [];
	var obj = function( pos, mass )
	{
		
		this.speed = 5;
		this.invMass = 1 / mass;
		
		this.pos = pos.clone();
		this.acc = new THREE.Vector3(0,0,0);
		this.vel = new THREE.Vector3(0,0,0);
		this.force = new THREE.Vector3(0,0,0);
		
		this.update = function()
		{
			
			var dx = 0,
				dy = 0,
				dz = 0;
			for( var i = 0; i < items.length; i++ )
			{
				
				this.force = items[ i ].vel.clone();
				this.force.normalize();
				//console.log( 'acc before ', this.acc );
				//console.log( 'force', this.force.x, this.force.y, this.force.z );
				
				var dist = items[ i ].pos.distanceTo( this.mesh.position );
				if( dist < 1 ) continue;
				
				var attenuation = ( items[ i ].strength / dist ) * this.invMass;
				
				this.force.multiplyScalar( attenuation );
				//dx += this.force.x;
				//dy += this.force.y;
				//dz += this.force.z;
				this.acc.add( this.force );
			}
			
			
			//console.log( 'acc after ', this.acc );
				
			//add gravity
			//this.acc.y += gravity;
			
			//this.vel.add( this.acc );
			//console.log( '->',this.acc, this.vel );
			
			//this.acc.multiplyScalar( friction );
			//console.log( 'acc', this.acc.x, this.acc.y, this.acc.z );
			
			this.acc.y = utils.clamp( this.acc.y, -this.speed, this.speed );
			this.acc.z = utils.clamp( this.acc.z, -this.speed, this.speed );
			this.acc.x = utils.clamp( this.acc.x, -this.speed, this.speed );
			
			this.mesh.position.add( this.acc );
			
			if( this.mesh.position.x < bbox.x ) this.mesh.position.x = bbox.x;
			if( this.mesh.position.y < bbox.y ) this.mesh.position.y = bbox.y;
			if( this.mesh.position.z < bbox.z ) this.mesh.position.z = bbox.z;
			if( this.mesh.position.x > bbox.w ) this.mesh.position.x = bbox.w;
			if( this.mesh.position.y > bbox.h ) this.mesh.position.y = bbox.h;
			if( this.mesh.position.z > bbox.d ) this.mesh.position.z = bbox.d;
			
		}
		
		this.dispose = function()
		{
			group.remove( this.mesh );
		}
		
		this.mesh = new THREE.Mesh( sphereGeom, white );
		this.mesh.position.copy( pos );
		this.mesh.scale.multiplyScalar( mass / 100 );
		group.add( this.mesh );
		
		objects.push( this );
		return this;
	}
	
	////////////////////////////////////////////////
	
	function createGrid()
	{
		while( items.length > 0 )
		{
			items[ 0 ].dispose();
			items.shift();
		}
		
		if( grid != null )
		{
			gridGeom.dispose();
			group.remove( grid );
		}
		
		gridGeom = new THREE.CubeGeometry( size, size, size, w-1, h-1, d-1 );
		gridMat = new THREE.MeshBasicMaterial( { color:0x606060, wireframe:true } );
		grid = new THREE.Mesh( gridGeom, gridMat );
		group.add( grid );
		
		var half = size / 2;
		var stepX = size / ( w - 1 );
		var stepY = size / ( h - 1 );
		var stepZ = size / ( d - 1 );
		
		for( var i = 0; i < w; i++ )
		{
			for( var j = 0; j < h; j++ )
			{
				for( var k = 0; k < d; k++ )
				{
				
					var pos = new THREE.Vector3( 	- half + i * stepX, 
													- half + j * stepY, 
													- half + k * stepZ );
													
					items.push( new force( pos, 10+Math.random() * 20 ) );
					
				}
			}
		}
	}
	
	function reset( _w, _h, _d )
	{
		var tmp = animation;
		animation = false;
		w = _w;
		h = _h;
		d = _d;
		createGrid();
		
		if( objects.length == 0 )
		{
			var count = 100;
			while( count-- ) 
			{
				new obj( new THREE.Vector3( ), 50 + ( Math.random() - .5 ) * 500 );
			}
		}
		objects.forEach( function( e, i, a )
		{	
			e.mesh.position.x = bbox.x + Math.random() * size;
			e.mesh.position.y = bbox.y + Math.random() * size,
			e.mesh.position.z = bbox.z + Math.random() * size;
			e.vel.set( 0, 0, 0 );
			e.acc.set( 0, 0, 0 );
		});
		
		animation = tmp;
		
	}
	reset( w, h, d );
	
	function update()
	{
		if( animation )
		{
			items.forEach( function( e, i, a ){	e.update();	});
			//items.forEach( function( e, i, a ){	console.log( e.vel );	});
			objects.forEach( function( e, i, a ){	e.update();	});
		}
	}
	setInterval( update, 1000 / 60 );
	
	setInterval( reset, 10000, w, h, d );
	
	function onDown( e )
	{
		//animation = true;
		//update();
		animation = !animation ? true : false;
	}
	window.addEventListener( 'mousedown', onDown );
	
	