/**
 * @author Nicolas Barradeau 
 */
		/* 
		http://www.joesfer.com/?p=46 
		http://algorithmicbotany.org/papers/colonization.egwnp2007.large.pdf
		*/
 
var branchesGeom, line;
var particleMat  = new THREE.ParticleBasicMaterial({ color:COLOR_GREY, size: .5} );
var particleGeom  = new THREE.Geometry();
var particleSystem = new THREE.ParticleSystem( particleGeom, particleMat );
group.add( particleSystem );

var imat = getMeshMaterial( COLOR_PINK, .15, false, BLEND_SUBTRACT );
var innerSphere = new THREE.Mesh( geom_sphere_2, imat );
innerSphere.scale.multiplyScalar( 119 );
group.add( innerSphere );

function createNode( pos, size )
{
	var m = new THREE.Mesh( geom_sphere_1, MAT_BLUE );
	m.position.copy( pos );
	m.scale.multiplyScalar( size );
	group.add( m );
	return m;
}

function rebuildCloud( attractors )
{
	if( particleSystem != null )
	{
		group.remove( particleSystem );
		particleGeom.dispose();
		delete particleGeom;
	}
	
	particleGeom  = new THREE.Geometry();
	particleSystem = new THREE.ParticleSystem( particleGeom, particleMat );
	group.add( particleSystem );
	
	for( var i = 0; i < attractors.length; i++ )
	{
		
		particleGeom.vertices.push( attractors[ i ].pos );
		
	}
	
}

var attractor = function( pos )
{
	this.pos 	= new THREE.Vector3();
	if( pos != null ) this.pos.copy( pos );
	
	this.id = particleGeom.vertices.length;
	particleGeom.vertices.push( this.pos );
	
	this.active = true;
	this.hooked = false;
	this.kill = function()
	{
		this.active = false;
	}
	return this;
}

var node = function( pos )
{
	
	this.pos 		= new THREE.Vector3();
	if( pos != null ) this.pos.copy( pos );
	
	this.m 			= createNode( this.pos, 1 );
	this.dir 		= new THREE.Vector3(0,0,0);
	this.path 		= [ this.pos.clone() ];
	this.attractors = [];
	this.active 	= true;
	
	this.averageDirection = function()
	{	
		for( var i = 0; i < this.attractors.length; i++ )
		{
			var t = new THREE.Vector3(0,0,0);
			t.copy( this.attractors[ i ].pos );
			t.sub( this.pos );
			this.dir.add( t );
		}
	}
	
	this.moveForward = function( distance )
	{	
		this.dir.normalize();
		this.dir.multiplyScalar( distance );
		
		this.pos.add( this.dir );
		this.path.push( this.pos.clone() );
	}
	
	this.rebuild = function()
	{
		for( var i = 0; i< this.path.length - 1; i++ )
		{
			branchesGeom.vertices.push( this.path[ i ] );
			branchesGeom.vertices.push( this.path[ i + 1 ] );
		}
	}
	
	return this;
}

var tree = function( nodes, attractors, searchRadius, killRadius, growDistance, maxNeighbors )
{
	
	this.nodes 					= nodes;
	this.attractors 			= attractors;
	this.searchRadius 			= searchRadius;
	this.growDistance 			= growDistance;
	this.killRadius 			= killRadius;
	this.maxNeighbors 			= maxNeighbors;
	this.lastCount 				= -1;
	this.lastCountIterator 		= 0;
	
	this.update = function()
	{
		
		if( this.attractors.length == this.lastCount )this.lastCountIterator++;
		if( this.attractors.length == 0 || this.lastCountIterator > 50 )
		{
			console.log( 'over', this.attractors.length  );
			return false;
		}
		
		////////////////////
		
		//flush the line set
		if( line != null )
		{
			group.remove( line );
			branchesGeom.dispose();
			delete branchesGeom;
		}
		
		branchesGeom = new THREE.Geometry();
		line = new THREE.Line( branchesGeom, LINE_BLUE );
		line.type = THREE.LinePieces;
		group.add( line );
	
		////////////////////
		
		var i, 
			j, 
			node,
			att;
			
		for( i = 0; i < this.nodes.length; i++ )
		{
			
			//find closest attractors to each node
			node = this.nodes[ i ];
			node.attractors = [];
			
			if( node.active )
			{
				for( j = 0; j < this.attractors.length; j++ )
				{
					att = this.attractors[ j ];
					
					if( !att.active || att.hooked == true ) continue;
					
					var dist = utils.dist( node.pos, att.pos );
					
					if( dist < this.searchRadius )
					{
						//att.hooked = true;
						node.attractors.push( att );
					}
					
					if( node.attractors.length >= this.maxNeighbors )
					{
						continue;
					}
						
				}
				
				//recompute direction if attractors found
				if( node.attractors.length != 0 )
				{
					
					//average direction
					node.averageDirection();
					
					//move forward by the growDistance amount
					node.moveForward( this.growDistance );
							
					//remove all attractors within the killRadius
					for ( j = 0; j < node.attractors.length; j++ )
					{
						att = node.attractors[ j ];
						if( utils.dist( node.pos, att.pos ) < this.killRadius )
						{
							att.kill();
						}
						//att.hooked = false;
					}
					
				}
				else
				{
					node.active = false;
				}
					
			}
			
			//rebuild new branches
			node.rebuild();
			
		}
		
		
		//flush the dead attractors
		var tmp = [];
		for( i = 0; i < this.attractors.length; i++ )
		{
			if( this.attractors[ i ].active )
			{
				tmp.push( this.attractors[ i ] );
			}
			else
			{
				particleGeom.vertices[ this.attractors[ i ].id ].setLength( 10000000 );//throw this vertex far far away 
			}
		}
		particleGeom.verticesNeedUpdate = true;
		
		this.attractors = tmp.concat();
		//rebuildCloud( tmp );
		
		//adds nodes
		if( Math.random() < respawnRate )
		{
			this.addRandomNode();
		}
		
		if( this.lastCount != this.attractors.length )this.lastCountIterator = 0;
		this.lastCount = this.attractors.length;
		
		return true;
		
	}
	
	this.addRandomNode = function()
	{	
		var id = Math.floor( Math.random() * this.nodes.length );
		var no = this.nodes[ id ];
		var nn = new node( no.pos );
		for( var i = 0; i < this.nodes.length; i++ )
		{
			this.nodes[ i ].m.scale.multiplyScalar( 1.01 );
		}
		
		this.nodes.push( nn );
	}
}
SSAO = true;
var i,
	p,
	debug = false,
	attractors = [],
	nodes = [],
	tree, 
	
	count = 10000;
	scale = 120,	

	att_distribution = 0,
	nod_distribution = 0,
	
	nodeCount = 1,
	respawnRate = .5,
	searchRadius = 16,
	killRadius = 12,
	growDistance = 3,
	maxNeighbors = 10;

	//att_distribution = 0,
	//nod_distribution = 0,
	//nodeCount = 4,
	//respawnRate = .25,
	//searchRadius = scale * .15,
	//killRadius = scale * .09,
	//growDistance = 2,
	//maxNeighbors = 25;
	_='';
	
	
for( i = 0; i < count; i++ )
{
	
	p = new THREE.Vector3( 	2 * ( Math.random() - .5 ),
							2 * ( Math.random() - .5 ),
							2 * ( Math.random() - .5 )		);
	
	switch( att_distribution )
	{
		case 0:
			p.normalize();
			p.multiplyScalar( scale );
		break;
		case 1:
			p.multiplyScalar( scale );
			break;
	}
	
	attractors.push( new attractor( p ) );
	
}

for( i = 0; i < nodeCount; i++ )
{
	
	p = new THREE.Vector3( 	0, -scale, 0	);
	p = new THREE.Vector3( 	0, 0, 0	);
	if( att_distribution == 0 )
	{
		
		p = new THREE.Vector3( 	Math.random() - .5, - 50 - i * 10, Math.random() - .5	);
		p = new THREE.Vector3( 		2 * ( Math.random() - .5 ),
									2 * ( Math.random() - .5 ),
									2 * ( Math.random() - .5 )	);
		p.normalize();
		p.multiplyScalar( scale  );
		
	}
	nodes.push( new node( p ) );
	
}

tree = new tree( nodes, attractors, searchRadius, killRadius, growDistance, maxNeighbors );

function update()
{
	if( !tree.update() ) clearInterval( interval );
}
var interval = setInterval( update, 1 );
