/**
 * ...
 * @author Nicolas Barradeau 
 */

var objects = [],
	radius = 100,
	quat = new THREE.Quaternion(),
	animation = true
	;
	
	var sphere = new THREE.Mesh( geom_sphere_3, getMeshMaterial( COLOR_CLEAR, .2 ) );
	sphere.scale.multiplyScalar( radius );
	group.add( sphere );

	var obj = function( angle, speed, hasTrail )
	{
		objects.push( this );
		
		this.angle = angle;
		this.speed = speed;
		this.pos = new THREE.Vector3();
		this.q = new THREE.Quaternion();
		
		if( hasTrail )
		{
			this.trail = [];
		}
		
		this.update = function()
		{
			
			this.angle.add( this.speed );
			
			this.q.setFromEuler( this.angle );
			this.pos.set( 1,0,0 );
			this.pos.applyQuaternion( this.q );
			this.pos.setLength( radius );
			
			this.mesh.position.copy( this.pos );
			
			if( this.trail != null )
			{
				this.trail.push( this.pos.clone() );
				this.updateTrail();
			}
		}
		
		this.updateTrail = function()
		{
			if( this.trail.length < 2 ) return;
			if( this.trail.length > 20 ) this.trail.shift();
			
			this.disposeTrail();
			
			this.geom = new THREE.Geometry();
			for( var i = 0; i < this.trail.length; i++ )
			{
				this.geom.vertices.push( this.trail[ i ] );
			}
			this.line = new THREE.Line( this.geom, LINE_BLUE );
			group.add( this.line );
			
		}
		
		this.dispose = function()
		{
			group.remove( this.mesh );
			delete this.mesh;
			if( this.trail != null )
			{
				this.disposeTrail();
				this.trail = [];
			}
		}
		
		this.disposeTrail = function()
		{
			if( this.line != null )
			{
				group.remove( this.line );
				this.geom.dispose();
				delete this.geom;
			}
		}
		
		this.mesh = new THREE.Mesh( geom_sphere_3, MAT_WHITE );
		this.mesh.scale.multiplyScalar( 2 );
		group.add( this.mesh );
		
		this.update();
		
		return this;
	}
	
	function reset( max, trail )
	{
		trail = trail || true;
		objects.forEach( function( e, i, a ){ e.dispose(); } );
		objects = [];
		for( var i = 0; i < max; i++ )
		{
			addObject( trail );
		}
	}
	
	function addObject( trail )
	{
		var a = new THREE.Vector3( 	Math.random() * Math.PI * 2,
									Math.random() * Math.PI * 2,
									Math.random() * Math.PI * 2 );
		
		var s = new THREE.Vector3( 	( .5 + Math.random() * 3 ) * DEG_TO_RAD,
									( .5 + Math.random() * 3 ) * DEG_TO_RAD,
									( .5 + Math.random() * 3 ) * DEG_TO_RAD		);
		new obj( a, s, trail );
		
	}
	
	reset( 1 );
	
	function update()
	{
		if( animation ) objects.forEach( function( e, i, a ){	e.update();	});
	}
	setInterval( update, 1000 / 30 );
	
	
	function onDown( e )
	{
		addObject( true );
	}
	window.addEventListener( 'mousedown', onDown );
	