/**
 * ...
 * @author Nicolas Barradeau 
 */

var objects = [],
	radius = 100,
	quat = new THREE.Quaternion(),
	links,
	linksGeom,
	center;	
	
	
	//var light = new THREE.PointLight( 0xff0000, 10, 100 );
	//light.position.set( 250, 250, 100 );
	//scene.add( light );

	var sphere = new THREE.Mesh( geom_sphere_3, getMeshMaterial( COLOR_CLEAR, .2 ) );
	sphere.scale.multiplyScalar( radius );
	group.add( sphere );
	
	

	var obj = function( angle, speed, hasTrail, material, trailMaterial )
	{
		objects.push( this );
		
		this.angle = angle || new THREE.Vector3();
		this.speed = speed || new THREE.Vector3();
		this.pos = new THREE.Vector3();
		this.q = new THREE.Quaternion();
		this.autonomous = true;
		this.length = 20;
		
		if( hasTrail )
		{
			this.trail = [];
			this.trailMaterial = trailMaterial || LINE_BLUE;
		}
		
		this.update = function()
		{
			if( this.autonomous )
			{
				this.angle.add( this.speed );
				
				this.q.setFromEuler( this.angle );
				
				this.pos.set( 1,0,0 );
				this.pos.applyQuaternion( this.q );
				this.pos.setLength( radius );
				
			}
			else
			{
				this.pos.set(0,0,0);
				for( var i = 0; i < objects.length; i++ )
				{
					if( objects[ i ] == this )continue;
					this.pos.add( objects[ i ].pos );
				}
				this.pos.multiplyScalar( 1 / ( objects.length - 1 ) );
				this.pos.setLength( radius );
			}
			
			this.mesh.position.copy( this.pos );
			
			if( this.trail != null )
			{
				this.trail.push( this.pos.clone() );
				this.updateTrail();
			}
		}
		
		this.updateTrail = function()
		{
			if( this.trail.length < 2 ) return;
			if( this.trail.length > this.length ) this.trail.shift();
			
			this.disposeTrail();
			
			this.geom = new THREE.Geometry();
			for( var i = 0; i < this.trail.length; i++ )
			{
				this.geom.vertices.push( this.trail[ i ] );
			}
			this.line = new THREE.Line( this.geom, this.trailMaterial );
			group.add( this.line );
			
		}
		
		this.dispose = function()
		{
			group.remove( this.mesh );
			delete this.mesh;
			if( this.trail != null )
			{
				this.disposeTrail();
				this.trail = [];
			}
		}
		
		this.disposeTrail = function()
		{
			if( this.line != null )
			{
				group.remove( this.line );
				this.geom.dispose();
				delete this.geom;
			}
		}
		
		this.mesh = new THREE.Mesh( geom_sphere_3, material || MAT_BLUE );
		this.mesh.scale.multiplyScalar( 1 );
		group.add( this.mesh );
		
		this.update();
		
		return this;
	}
	
	function reset( max, trail )
	{
		trail = trail || false;
		objects.forEach( function( e, i, a ){ e.dispose(); } );
		
		objects = [];
		for( var i = 0; i < max; i++ )
		{
			addObject( trail );
		}
		
		center = new obj( null, null, true, LINE_YELLOW, MAT_YELLOW );
		center.autonomous = false;
		center.length = 200;
		center.mesh.scale.multiplyScalar( 2 );
		center.update();
		center.trail = [];
		
		
	}
	
	function addObject( trail )
	{
		var a = new THREE.Vector3( 	Math.random() * Math.PI * 2,
									Math.random() * Math.PI * 2,
									Math.random() * Math.PI * 2 );
		
		var s = new THREE.Vector3( 	( .5 + Math.random() * 3 ) * DEG_TO_RAD,
									( .5 + Math.random() * 3 ) * DEG_TO_RAD,
									( .5 + Math.random() * 3 ) * DEG_TO_RAD		);
		new obj( a, s, trail );
		
	}
	
	function update()
	{
		
		objects.forEach( function( e, i, a ){	e.update();	});
		updateLinks();
		
	}
	setInterval( update, 1000 / 30 );
	
	function updateLinks()
	{
		
		if( objects.length == 0 ) return;
		
		if( links != null )
		{
			group.remove( links );
			linksGeom.dispose();
			delete linksGeom;
		}
		
		var c = new THREE.Vector3();
		linksGeom = new THREE.Geometry();
		
		for( var i = 0; i < objects.length; i++ )
		{
			if( objects[ i ] == center )continue;
			c.add( objects[ i ].pos );
			
		}
		c.multiplyScalar( 1 / ( objects.length - 1 ) );
		linksGeom.vertices.push( c );
		linksGeom.vertices.push( center.pos );
		
		for( var i = 0; i < objects.length; i++ )
		{
			if( objects[ i ] == center )continue;
			//linksGeom.vertices.push( c );
			//linksGeom.vertices.push( objects[ i ].pos );
			
			linksGeom.vertices.push( center.pos );
			linksGeom.vertices.push( objects[ i ].pos );
		}
		
		links = new THREE.Line( linksGeom, LINE_GREY );
		group.add( links );
		
	}
	
	function onDown( e )
	{
		if( objects.length > 10 )
		{
			reset( 1, true );
		}
		else
		{
			center.trail = [];
			addObject( true );
		}
	}
	window.addEventListener( 'mousedown', onDown );
	
	reset( 1, true );
	