/**
 * @author Nicolas Barradeau 
 */
var g = new THREE.IcosahedronGeometry( 5, 1 );
var clear = new THREE.MeshBasicMaterial({color:0xEEEEEE} );
var lmat = new THREE.LineBasicMaterial( { color:0xFFFFFF } );
	
var Turtle = function( pos, rot )
{
	
	this.pos = pos || new THREE.Vector3();
	this.x = this.pos.x;
	this.y = this.pos.y;
	this.z = this.pos.z;
	this.rot = rot || new THREE.Vector3();
	
	this.q = new THREE.Quaternion();
	this.q.setFromEuler( this.rot );
	
	this.dir = new THREE.Vector3(1,0,0);
	this.dir.applyQuaternion( this.q );
	
	this.clone = function()
	{
		return new Turtle( this.pos.clone(), this.rot.clone() );
	}
	
	this.forward = function( length )
	{
		
		this.q.setFromEuler( this.rot );
		this.q.normalize();
		
		this.dir.applyQuaternion( this.q );
		this.dir.normalize();
		this.dir.multiplyScalar( length );
		
		this.pos = this.pos.add( this.dir );
		
	}
	
	this.update = function()
	{
		this.x = this.pos.x;
		this.y = this.pos.y;
		this.z = this.pos.z;
	}
	
	this.rx = function( angle ){	this.rot.x += angle;		}
	this.ry = function( angle ){	this.rot.y += angle;		}
	this.rz = function( angle ){	this.rot.z += angle;		}
	
	return this;
}


var turtle = new Turtle();
var axiom = "";
var rules = [];
rules[ "f" ] = "f+f-f-f";
rules[ "a" ] = "-fbf+";
rules[ "b" ] = ",fcf;";
rules[ "c" ] = "$faf$";

var line, 
	geom,
	delta = Math.PI / 45,
	length = 2;

function rewrite()
{
	
	var new_axiom = "";
	
	var tmp = axiom.split( '' );
	
	for( var i = 0; i < tmp.length; i++ )
	{
		var character = tmp[ i ];
		new_axiom += rules[ character ];
	}


	for( i = 0; i < new_axiom.length; i++ )
	{
		
		character = new_axiom.charAt( i ).toLowerCase();
		
		if( character == '[' )
		{
			if ( new_axiom.substr( i+1, new_axiom.length ).lastIndexOf( ']' ) == -1 ) continue;
			
			var tmp_turtle = turtle.clone();
			while ( character != ']')
			{
				character = new_axiom.charAt( i++ );
				process( character, tmp_turtle );
			}
			
		}
		else
		{
			process( character, turtle );
		}
		
	}
	
	axiom = new_axiom;
	return axiom;
}




function process( character, turtle )
{
	
	switch( character )
	{
		//move forward
		case "f":
			turtle.forward( length );
			break;
			
		// angle X
		case "$":		turtle.rx( delta );			break;
		case "£":		turtle.rx( -delta );		break;
			
		// angle Y
		case ",":		turtle.ry( delta );			break;
		case ";":		turtle.ry( -delta );		break;
			
		// angle Z
		case "+":		turtle.rz( delta );			break;
		case "-":		turtle.rz( -delta );		break;
	}
	turtle.update();
	geom.vertices.push( new THREE.Vector3( turtle.x, turtle.y, turtle.z ) );
	
}

var samples = [ /* axiom, 	delta, length, iterations */
				['$fbf,,fbaaf-+', Math.PI / 90, 1, 4 ],  
				['af$;-$[ffc;],f]', Math.PI / 90, 1, 4 ],  
				['b,£ff+f£f', Math.PI / 90, 1, 4 ],  
				['+ffbc,$ffcb-;', Math.PI / 90, 1, 4 ],  
				[',fb£+cf[[+-f]f]', Math.PI / 90, 2, 4 ],  
				
				['£+bf[;f£cf$-,f]', Math.PI / 45, 2, 4 ],  
				['+b-[+f-;]]', Math.PI / 45, 2, 4 ],  
				
				];

function reset( count )
{
	count = count || 10;
	
	var openBrackets = 0;
	var closeBrackets = 0;
	
	var alphabet 	= [ 'f', 'a', 'b', 'c', '-', '+', '[', ']', '$', '£', ',', ';' ];
	var tmp = '';
	for ( var i = 0; i < count; i++) 
	{
		var symbolindex = Math.floor( Math.random() * alphabet.length );
		
		var letter = alphabet[ symbolindex ];
		
		if ( letter == '[' )
		{
			openBrackets++;
		}
		if ( letter == ']' && openBrackets == 0 )continue;
		if ( letter == ']' )
		{
			closeBrackets--;
		}
		
		tmp += letter;
	}
	
	if ( openBrackets > 0) 
	{
		for ( i = 0; i < openBrackets; i++)
		{
			tmp += ']';
		}
	}
	if( tmp.lastIndexOf( 'f' ) < 4 ) 
	{
		for( i = 0; i < 4; i++ )
		{
			var index = Math.floor( Math.random() * tmp.length );
			tmp = tmp.substr( 0,index ) + 'f' + tmp.substr( index );
		}
	}
	console.log(tmp);
	
	axiom = tmp;
	
	return axiom;
}

function compute( count )
{
	count = count|| 4;
	if( line != null )
	{
		group.remove( line );
		geom.dispose();
	}

	geom = new THREE.Geometry();
	line = new THREE.Line( geom, lmat );
	group.add( line );
	
	while( count-- )rewrite();
	
	//geom.computeBoundingSphere();
	//geom.computeBoundingBox();
	//camera.position.z = geom.boundingSphere.radius * .5;
	
}

function onDown( e )
{ 
	reset( 10 );
	compute();
}
reset( 10 );
compute();

window.addEventListener( 'mousedown', onDown );