/**
 * ...
 * @author Nicolas Barradeau 
 */

//(function() {
			
	// workaround for chrome bug: http://code.google.com/p/chromium/issues/detail?id=35980#c12
	if ( window.innerWidth === 0 ) { window.innerWidth = parent.innerWidth; window.innerHeight = parent.innerHeight; }
	
	var camera, 
		scene, 
		renderer,
		group,
		depthMaterial, 
		depthTarget, 
		composer,
		effect,
		camdest = 500,
		width = 600,
		height = 400,
		mouseX = 0,
		mouseY = 0,
		mouseDown = false,
		started = false,
		iteration = 0,
		max = 20;
	
	var items = [];
	
	
	///////////////////////////////////////////////////
	
	
	init();
	animate();
	
	function init() {

		var bgColor = 0x000000;
		var fgColor = 0x303030;
	
		renderer = new THREE.WebGLRenderer();
		renderer.setSize( window.innerWidth, window.innerHeight );
		renderer.setClearColorHex( bgColor, 1 );
		document.body.appendChild( renderer.domElement );
		
		camera = new THREE.PerspectiveCamera( 36, window.innerWidth / window.innerHeight, 10, 10000 );
		camera.position.y = 0;
		camera.position.z = camdest;

		scene = new THREE.Scene();
		scene.fog = new THREE.Fog( bgColor, .1, 1500 );

		group = new THREE.Object3D();
		scene.add( group );
		
		//ground
		var geometry = new THREE.PlaneGeometry( 10, 10, 1, 1 );
		var material = new THREE.MeshBasicMaterial( { color: fgColor, side:THREE.DoubleSide } );
		ground = new THREE.Mesh( geometry, material );
		ground.rotation.x = -Math.PI / 2;
		ground.position.y = -300;
		ground.scale.x = ground.scale.y = 10000;
		group.add( ground );
		
		// depth
		var depthShader = THREE.ShaderLib[ "depthRGBA" ];
		var depthUniforms = THREE.UniformsUtils.clone( depthShader.uniforms );
		depthMaterial = new THREE.ShaderMaterial( { fragmentShader: depthShader.fragmentShader, vertexShader: depthShader.vertexShader, uniforms: depthUniforms } );
		depthMaterial.blending = THREE.NoBlending;

		// postprocessing
		composer = new THREE.EffectComposer( renderer );
		composer.addPass( new THREE.RenderPass( scene, camera ) );

		depthTarget = new THREE.WebGLRenderTarget( window.innerWidth, window.innerHeight, { minFilter: THREE.NearestFilter, magFilter: THREE.NearestFilter, format: THREE.RGBAFormat } );
		
		//SSAO
		effect = new THREE.ShaderPass( THREE.SSAOShader );
		effect.uniforms[ 'tDepth' ].value = depthTarget;
		effect.uniforms[ 'size' ].value.set( window.innerWidth, window.innerHeight );
		effect.uniforms[ 'cameraNear' ].value = camera.near;
		effect.uniforms[ 'cameraFar' ].value = camera.far;
		effect.renderToScreen = true;
		composer.addPass( effect );
		
		//handlers
		window.onresize = onResize;
		window.onmousemove = onMouseMove;
		window.onmousewheel = onMouseWheel;
		onResize();
		
	}

	function onMouseMove(e)
	{
		mouseX = ( e.clientX - width / 2  ) / width;
		mouseY = ( e.clientY - height / 2 ) / height;
	}
	function onMouseWheel(e)
	{
		var delta = 0;
		if (!e)e = window.event;
        if ( e.wheelDelta){  delta = e.wheelDelta/120;
        } else if ( e.detail){ delta = -e.detail/3;	}
		camdest += delta * 25;
	}
	 
	function onResize( e )
	{
		
		width = window.innerWidth;
		height = window.innerHeight;
		
		renderer.setSize( width, height );
		camera.aspect	= width / height;
		camera.updateProjectionMatrix();
		
	}
	
	function checkMax()
	{
		if( group.children.length >= max ) 
		{
			while( group.children.length > 0 && items[0] != null ) 
			{
				group.remove( items[0].mesh );
				items.shift();
			}
			return;
		}
	}
	var SSAO = false;
	function animate() {

		if( camdest < 5 ) camdest = 5;
		camera.position.z += ( camdest - camera.position.z ) * .1;
		
		var timer = performance.now();
		
		group.rotation.y += ( mouseX * 4	- group.rotation.y ) * .1;//(- group.rotation.y ) * .1;
		group.rotation.x += ( mouseY * .75	- group.rotation.x ) * .1;//(- group.rotation.y ) * .1;
		
		if( SSAO )
		{
			//console.log( 'SSAO = true' );
			scene.overrideMaterial = depthMaterial;
			renderer.render( scene, camera, depthTarget );
			scene.overrideMaterial = null;
			composer.render();
		}
		else
		{
			renderer.render( scene, camera );
		}
		
		requestAnimationFrame( animate );
		
	}
	/*****************************************************************
	
	UTILS / CONSTANTS
	
	*****************************************************************/
	
	var DEG_TO_RAD		= Math.PI / 180;
	var RAD_TO_DEG		= 180 / Math.PI;
	
	var COLOR_WHITE 	= 0xFFFFFF;
	var COLOR_GREY 		= 0x808080;
	var COLOR_BLACK 	= 0x000000;
	var COLOR_RED 		= 0xDD0000;
	var COLOR_GREEN 	= 0x00CC33;
	var COLOR_YELLOW 	= 0xFFAA00;
	var COLOR_BLUE 		= 0x006699;
	var COLOR_PINK 		= 0xFF0066;
	
	var COLOR_CLEAR 	= 0xEEEEEE;
	var COLOR_DARK 		= 0x303030;
	
	var BLEND_NONE 		= 	THREE[ "NoBlending"				];
	var BLEND_NORMAL 	= 	THREE[ "NormalBlending"			];
	var BLEND_ADD 		=	THREE[ "AdditiveBlending"		];
	var BLEND_SUBTRACT 	= 	THREE[ "SubtractiveBlending"	];
	var BLEND_MULTIPLY 	= 	THREE[ "MultiplyBlending"		];
	var BLEND_ADD_ALPHA = 	THREE[ "AdditiveAlphaBlending" 	];
	
	function getLineMaterial( _color, _alpha, _blending )
	{
		
		var l_mat;
		var c = _color || ( 0x808080 + Math.random() * 0x808080 );
		var a = _alpha || 1;
		var t = a == 1 ? false : true;
		var b = _blending || BLEND_NORMAL;
		return  new THREE.LineBasicMaterial( { color:_color, opacity:a, transparent:t, blending:b } );
		
	}
	
	var LINE_CLEAR 		= getLineMaterial( COLOR_CLEAR	);
	var LINE_DARK 		= getLineMaterial( COLOR_DARK 	);
	var LINE_WHITE 		= getLineMaterial( COLOR_WHITE 	);
	var LINE_GREY 		= getLineMaterial( COLOR_GREY 	);	
	var LINE_BLACK 		= getLineMaterial( COLOR_BLACK 	);
	var LINE_RED 		= getLineMaterial( COLOR_RED 	);	
	var LINE_GREEN 		= getLineMaterial( COLOR_GREEN 	);
	var LINE_YELLOW 	= getLineMaterial( COLOR_YELLOW );	
	var LINE_BLUE 		= getLineMaterial( COLOR_BLUE 	);	
	var LINE_PINK 		= getLineMaterial( COLOR_PINK 	);	
	
	function getMeshMaterial( _color, _alpha, _wireframe, _blending )
	{
		
		var l_mat;
		var c = _color || ( 0x808080 + Math.random() * 0x808080 );
		var a = _alpha || 1;
		var t = a == 1 ? false : true;
		var w = _wireframe ? true : false;
		var b = _blending || BLEND_NORMAL;
		
		return  new THREE.MeshBasicMaterial( { color:_color, opacity:a, transparent:t, wireframe:w, blending:b } );
		
	}
	
	var MAT_CLEAR 		= getMeshMaterial( COLOR_CLEAR 	);
	var MAT_DARK 		= getMeshMaterial( COLOR_DARK  	);
	var MAT_WHITE 		= getMeshMaterial( COLOR_WHITE 	);
	var MAT_GREY 		= getMeshMaterial( COLOR_GREY 	);	
	var MAT_BLACK 		= getMeshMaterial( COLOR_BLACK 	);
	var MAT_RED 		= getMeshMaterial( COLOR_RED 	);	
	var MAT_GREEN 		= getMeshMaterial( COLOR_GREEN 	);
	var MAT_YELLOW 		= getMeshMaterial( COLOR_YELLOW );	
	var MAT_BLUE 		= getMeshMaterial( COLOR_BLUE 	);	
	var MAT_PINK 		= getMeshMaterial( COLOR_PINK 	);	
	
	
	var geom_sphere_0 = new THREE.IcosahedronGeometry( 1,0 );
	var geom_sphere_1 = new THREE.IcosahedronGeometry( 1,1 );
	var geom_sphere_2 = new THREE.IcosahedronGeometry( 1,2 );
	var geom_sphere_3 = new THREE.IcosahedronGeometry( 1,3 );
	
	var geom_cube = new THREE.CubeGeometry( 1,1,1,1,1,1 );
	var geom_cylinder = new THREE.CylinderGeometry( 1,1,1 );
	
	
	
	var utils = function(){};
	utils.sqdist = function( a, b ){ var dx = a.x - b.x; var dy = a.y - b.y; if( a.z != null ){ var dz = a.z - b.z; return (dx*dx+dy*dy+dz*dz); }else{ return (dx*dx+dy*dy);} };
	utils.dist = function( a, b ){ return Math.sqrt( utils.sqdist(a, b) ); };
	utils.lerp = function( t, a, b ){ return a + t * ( b - a ); };
	utils.norm = function( t, a, b ){ return ( t - a ) / ( b - a ); };
	utils.map = function( t, a0, b0, a1, b1 ){ return utils.lerp( utils.norm( t, a0, b0 ), a1, b1 ); };
	utils.clamp = function( t, min, max ){ return Math.min( max, Math.max( t, min ) ); };
	
	window.oncontextmenu = function ()
	{
		window.close();
		return false;
	}
	
//})();